﻿//	Copyright (c) 2012, Michael Kunz. All rights reserved.
//	http://managedcuda.codeplex.com
//
//	This file is part of ManagedCuda.
//
//	ManagedCuda is free software: you can redistribute it and/or modify
//	it under the terms of the GNU Lesser General Public License as 
//	published by the Free Software Foundation, either version 2.1 of the 
//	License, or (at your option) any later version.
//
//	ManagedCuda is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//	GNU Lesser General Public License for more details.
//
//	You should have received a copy of the GNU Lesser General Public
//	License along with this library; if not, write to the Free Software
//	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
//	MA 02110-1301  USA, http://www.gnu.org/licenses/.


using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using ManagedCuda.BasicTypes;
using ManagedCuda.VectorTypes;


namespace ManagedCuda.CudaSparse
{
	/// <summary>
	/// C# wrapper for cusparse_v2.h
	/// </summary>
	public static class CudaSparseNativeMethods
	{
		//unfortunately Nvidia provides different dll-names for x86 and x64. Use preprocessor macro to switch names:
#if _x64
		internal const string CUSPARSE_API_DLL_NAME = "cusparse64_55";
#else
		internal const string CUSPARSE_API_DLL_NAME = "cusparse32_55";
#endif
		#region CUSPARSE initialization and managment routines
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCreate(ref cusparseContext handle);
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDestroy(cusparseContext handle);
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseGetVersion(cusparseContext handle, ref int version);
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseSetStream(cusparseContext handle, CUstream streamId);
		#endregion

		#region CUSPARSE type creation, destruction, set and get routines

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseGetPointerMode(cusparseContext handle, ref cusparsePointerMode mode);
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseSetPointerMode(cusparseContext handle, cusparsePointerMode mode);
		#endregion

		#region sparse matrix descriptor
		/* When the matrix descriptor is created, its fields are initialized to: 
		CUSPARSE_MATRIXYPE_GENERAL
		CUSPARSE_INDEX_BASE_ZERO
		All other fields are uninitialized
		*/
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCreateMatDescr(ref cusparseMatDescr descrA);
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDestroyMatDescr(cusparseMatDescr descrA);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseSetMatType(cusparseMatDescr descrA, cusparseMatrixType type);
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseMatrixType cusparseGetMatType(cusparseMatDescr descrA);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseSetMatFillMode(cusparseMatDescr descrA, cusparseFillMode fillMode);
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseFillMode cusparseGetMatFillMode(cusparseMatDescr descrA);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseSetMatDiagType(cusparseMatDescr descrA, cusparseDiagType diagType);
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseDiagType cusparseGetMatDiagType(cusparseMatDescr descrA);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseSetMatIndexBase(cusparseMatDescr descrA, cusparseIndexBase indexBase);
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseIndexBase cusparseGetMatIndexBase(cusparseMatDescr descrA);
		#endregion

		#region sparse traingular solve
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCreateSolveAnalysisInfo(ref cusparseSolveAnalysisInfo info);
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDestroySolveAnalysisInfo(cusparseSolveAnalysisInfo info);
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseGetLevelInfo(cusparseContext handle, 
                                                  cusparseSolveAnalysisInfo info, 
                                                  ref int nlevels,
												  ref CUdeviceptr levelPtr,
												  ref CUdeviceptr levelInd);
		#endregion

		#region hybrid (HYB) format
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCreateHybMat(ref cusparseHybMat hybA);
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDestroyHybMat(cusparseHybMat hybA);
		#endregion

		#region Sparse Level 1 routines
		/* Description: Addition of a scalar multiple of a sparse vector x  
		and a dense vector y. */
		#region ref host
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseSaxpyi_v2(cusparseContext handle, int nnz, ref float alpha, CUdeviceptr xVal, CUdeviceptr xInd, CUdeviceptr y, cusparseIndexBase idxBase);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDaxpyi_v2(cusparseContext handle, int nnz, ref double alpha, CUdeviceptr xVal, CUdeviceptr xInd, CUdeviceptr y, cusparseIndexBase idxBase);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCaxpyi_v2(cusparseContext handle, int nnz, ref cuFloatComplex alpha, CUdeviceptr xVal, CUdeviceptr xInd, CUdeviceptr y, cusparseIndexBase idxBase);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZaxpyi_v2(cusparseContext handle, int nnz, ref cuDoubleComplex alpha, CUdeviceptr xVal, CUdeviceptr xInd, CUdeviceptr y, cusparseIndexBase idxBase);
		#endregion
		#region ref device
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseSaxpyi_v2(cusparseContext handle, int nnz, CUdeviceptr alpha, CUdeviceptr xVal, CUdeviceptr xInd, CUdeviceptr y, cusparseIndexBase idxBase);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDaxpyi_v2(cusparseContext handle, int nnz, CUdeviceptr alpha, CUdeviceptr xVal, CUdeviceptr xInd, CUdeviceptr y, cusparseIndexBase idxBase);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCaxpyi_v2(cusparseContext handle, int nnz, CUdeviceptr alpha, CUdeviceptr xVal, CUdeviceptr xInd, CUdeviceptr y, cusparseIndexBase idxBase);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZaxpyi_v2(cusparseContext handle, int nnz, CUdeviceptr alpha, CUdeviceptr xVal, CUdeviceptr xInd, CUdeviceptr y, cusparseIndexBase idxBase);
		#endregion

		/* Description: dot product of a sparse vector x and a dense vector y. */
		#region ref host
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseSdoti(cusparseContext handle, int nnz, CUdeviceptr xVal, CUdeviceptr xInd, CUdeviceptr y, ref float resultDevHostPtr, cusparseIndexBase idxBase);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDdoti(cusparseContext handle, int nnz, CUdeviceptr xVal, CUdeviceptr xInd, CUdeviceptr y, ref double resultDevHostPtr, cusparseIndexBase idxBase);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCdoti(cusparseContext handle, int nnz, CUdeviceptr xVal, CUdeviceptr xInd, CUdeviceptr y, ref cuFloatComplex resultDevHostPtr, cusparseIndexBase idxBase);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZdoti(cusparseContext handle, int nnz, CUdeviceptr xVal, CUdeviceptr xInd, CUdeviceptr y, ref cuDoubleComplex resultDevHostPtr, cusparseIndexBase idxBase);
		#endregion
		#region ref device
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseSdoti(cusparseContext handle, int nnz, CUdeviceptr xVal, CUdeviceptr xInd, CUdeviceptr y, CUdeviceptr resultDevHostPtr, cusparseIndexBase idxBase);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDdoti(cusparseContext handle, int nnz, CUdeviceptr xVal, CUdeviceptr xInd, CUdeviceptr y, CUdeviceptr resultDevHostPtr, cusparseIndexBase idxBase);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCdoti(cusparseContext handle, int nnz, CUdeviceptr xVal, CUdeviceptr xInd, CUdeviceptr y, CUdeviceptr resultDevHostPtr, cusparseIndexBase idxBase);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZdoti(cusparseContext handle, int nnz, CUdeviceptr xVal, CUdeviceptr xInd, CUdeviceptr y, CUdeviceptr resultDevHostPtr, cusparseIndexBase idxBase);
		#endregion

		/* Description: dot product of complex conjugate of a sparse vector x
		and a dense vector y. */
		#region ref host
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCdotci(cusparseContext handle, int nnz, CUdeviceptr xVal, CUdeviceptr xInd, CUdeviceptr y, ref cuFloatComplex resultDevHostPtr, cusparseIndexBase idxBase);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZdotci(cusparseContext handle, int nnz, CUdeviceptr xVal, CUdeviceptr xInd, CUdeviceptr y, ref cuDoubleComplex resultDevHostPtr, cusparseIndexBase idxBase);
		#endregion
		#region ref device
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCdotci(cusparseContext handle, int nnz, CUdeviceptr xVal, CUdeviceptr xInd, CUdeviceptr y, CUdeviceptr resultDevHostPtr, cusparseIndexBase idxBase);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZdotci(cusparseContext handle, int nnz, CUdeviceptr xVal, CUdeviceptr xInd, CUdeviceptr y, CUdeviceptr resultDevHostPtr, cusparseIndexBase idxBase);
		#endregion


		/* Description: Gather of non-zero elements from dense vector y into 
		sparse vector x. */
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseSgthr(cusparseContext handle, int nnz, CUdeviceptr y, CUdeviceptr xVal, CUdeviceptr xInd, cusparseIndexBase idxBase);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDgthr(cusparseContext handle, int nnz, CUdeviceptr y, CUdeviceptr xVal, CUdeviceptr xInd, cusparseIndexBase idxBase);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCgthr(cusparseContext handle, int nnz, CUdeviceptr y, CUdeviceptr xVal, CUdeviceptr xInd, cusparseIndexBase idxBase);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZgthr(cusparseContext handle, int nnz, CUdeviceptr y, CUdeviceptr xVal, CUdeviceptr xInd, cusparseIndexBase idxBase);

		/* Description: Gather of non-zero elements from desne vector y into 
		sparse vector x (also replacing these elements in y by zeros). */
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseSgthrz(cusparseContext handle, int nnz, CUdeviceptr y, CUdeviceptr xVal, CUdeviceptr xInd, cusparseIndexBase idxBase);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDgthrz(cusparseContext handle, int nnz, CUdeviceptr y, CUdeviceptr xVal, CUdeviceptr xInd, cusparseIndexBase idxBase);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCgthrz(cusparseContext handle, int nnz, CUdeviceptr y, CUdeviceptr xVal, CUdeviceptr xInd, cusparseIndexBase idxBase);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZgthrz(cusparseContext handle, int nnz, CUdeviceptr y, CUdeviceptr xVal, CUdeviceptr xInd, cusparseIndexBase idxBase);

		/* Description: Scatter of elements of the sparse vector x into 
		dense vector y. */
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseSsctr(cusparseContext handle, int nnz, CUdeviceptr xVal, CUdeviceptr xInd, CUdeviceptr y, cusparseIndexBase idxBase);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDsctr(cusparseContext handle, int nnz, CUdeviceptr xVal, CUdeviceptr xInd, CUdeviceptr y, cusparseIndexBase idxBase);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCsctr(cusparseContext handle, int nnz, CUdeviceptr xVal, CUdeviceptr xInd, CUdeviceptr y, cusparseIndexBase idxBase);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZsctr(cusparseContext handle, int nnz, CUdeviceptr xVal, CUdeviceptr xInd, CUdeviceptr y, cusparseIndexBase idxBase);

		/* Description: Givens rotation, where c and s are cosine and sine, 
		x and y are sparse and dense vectors, respectively. */
		#region ref host
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseSroti_v2(cusparseContext handle, int nnz, CUdeviceptr xVal, CUdeviceptr xInd, CUdeviceptr y, ref float c, ref float s, cusparseIndexBase idxBase);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDroti_v2(cusparseContext handle, int nnz, CUdeviceptr xVal, CUdeviceptr xInd, CUdeviceptr y, ref double c, ref double s, cusparseIndexBase idxBase);
		#endregion
		#region ref device
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseSroti_v2(cusparseContext handle, int nnz, CUdeviceptr xVal, CUdeviceptr xInd, CUdeviceptr y, CUdeviceptr c, CUdeviceptr s, cusparseIndexBase idxBase);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDroti_v2(cusparseContext handle, int nnz, CUdeviceptr xVal, CUdeviceptr xInd, CUdeviceptr y, CUdeviceptr c, CUdeviceptr s, cusparseIndexBase idxBase);
		#endregion

		#endregion

		#region Sparse Level 2 routines

		/* Description: Matrix-vector multiplication  y = alpha * op(A) * x  + beta * y, 
   where A is a sparse matrix in CSR storage format, x and y are dense vectors. */
		#region ref host
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseScsrmv_v2(cusparseContext handle, cusparseOperation transA, int m, int n, int nnz, ref float alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, CUdeviceptr x, ref float beta, CUdeviceptr y);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDcsrmv_v2(cusparseContext handle, cusparseOperation transA, int m, int n, int nnz, ref double alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, CUdeviceptr x, ref double beta, CUdeviceptr y);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCcsrmv_v2(cusparseContext handle, cusparseOperation transA, int m, int n, int nnz, ref cuFloatComplex alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, CUdeviceptr x, ref cuFloatComplex beta, CUdeviceptr y);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZcsrmv_v2(cusparseContext handle, cusparseOperation transA, int m, int n, int nnz, ref cuDoubleComplex alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, CUdeviceptr x, ref cuDoubleComplex beta, CUdeviceptr y);
		#endregion
		#region ref device
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseScsrmv_v2(cusparseContext handle, cusparseOperation transA, int m, int n, int nnz, CUdeviceptr alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, CUdeviceptr x, CUdeviceptr beta, CUdeviceptr y);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDcsrmv_v2(cusparseContext handle, cusparseOperation transA, int m, int n, int nnz, CUdeviceptr alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, CUdeviceptr x, CUdeviceptr beta, CUdeviceptr y);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCcsrmv_v2(cusparseContext handle, cusparseOperation transA, int m, int n, int nnz, CUdeviceptr alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, CUdeviceptr x, CUdeviceptr beta, CUdeviceptr y);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZcsrmv_v2(cusparseContext handle, cusparseOperation transA, int m, int n, int nnz, CUdeviceptr alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, CUdeviceptr x, CUdeviceptr beta, CUdeviceptr y);
		#endregion

		/* Description: Matrix-vector multiplication  y = alpha * op(A) * x  + beta * y, 
		   where A is a sparse matrix in HYB storage format, x and y are dense vectors. */
		#region ref host
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseShybmv(cusparseContext handle, cusparseOperation transA, ref float alpha, cusparseMatDescr descrA, cusparseHybMat hybA, CUdeviceptr x, ref float beta, CUdeviceptr y);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDhybmv(cusparseContext handle, cusparseOperation transA, ref double alpha, cusparseMatDescr descrA, cusparseHybMat hybA, CUdeviceptr x, ref double beta, CUdeviceptr y);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseChybmv(cusparseContext handle, cusparseOperation transA, ref cuFloatComplex alpha, cusparseMatDescr descrA, cusparseHybMat hybA, CUdeviceptr x, ref cuFloatComplex beta, CUdeviceptr y);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZhybmv(cusparseContext handle, cusparseOperation transA, ref cuDoubleComplex alpha, cusparseMatDescr descrA, cusparseHybMat hybA, CUdeviceptr x, ref cuDoubleComplex beta, CUdeviceptr y);
		#endregion
		#region ref device
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseShybmv(cusparseContext handle, cusparseOperation transA, CUdeviceptr alpha, cusparseMatDescr descrA, cusparseHybMat hybA, CUdeviceptr x, CUdeviceptr beta, CUdeviceptr y);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDhybmv(cusparseContext handle, cusparseOperation transA, CUdeviceptr alpha, cusparseMatDescr descrA, cusparseHybMat hybA, CUdeviceptr x, CUdeviceptr beta, CUdeviceptr y);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseChybmv(cusparseContext handle, cusparseOperation transA, CUdeviceptr alpha, cusparseMatDescr descrA, cusparseHybMat hybA, CUdeviceptr x, CUdeviceptr beta, CUdeviceptr y);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZhybmv(cusparseContext handle, cusparseOperation transA, CUdeviceptr alpha, cusparseMatDescr descrA, cusparseHybMat hybA, CUdeviceptr x, CUdeviceptr beta, CUdeviceptr y);
		#endregion

		/* Description: Solution of triangular linear system op(A) * y = alpha * x, 
		   where A is a sparse matrix in CSR storage format, x and y are dense vectors. */
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseScsrsv_analysis_v2(cusparseContext handle, cusparseOperation transA, int m, int nnz, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, cusparseSolveAnalysisInfo info);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDcsrsv_analysis_v2(cusparseContext handle, cusparseOperation transA, int m, int nnz, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, cusparseSolveAnalysisInfo info);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCcsrsv_analysis_v2(cusparseContext handle, cusparseOperation transA, int m, int nnz, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, cusparseSolveAnalysisInfo info);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZcsrsv_analysis_v2(cusparseContext handle, cusparseOperation transA, int m, int nnz, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, cusparseSolveAnalysisInfo info);


		#region ref host
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseScsrsv_solve_v2(cusparseContext handle, cusparseOperation transA, int m, ref float alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, cusparseSolveAnalysisInfo info, CUdeviceptr x, CUdeviceptr y);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDcsrsv_solve_v2(cusparseContext handle, cusparseOperation transA, int m, ref double alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, cusparseSolveAnalysisInfo info, CUdeviceptr x, CUdeviceptr y);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCcsrsv_solve_v2(cusparseContext handle, cusparseOperation transA, int m, ref cuFloatComplex alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, cusparseSolveAnalysisInfo info, CUdeviceptr x, CUdeviceptr y);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZcsrsv_solve_v2(cusparseContext handle, cusparseOperation transA, int m, ref cuDoubleComplex alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, cusparseSolveAnalysisInfo info, CUdeviceptr x, CUdeviceptr y);
		#endregion
		#region ref device
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseScsrsv_solve_v2(cusparseContext handle, cusparseOperation transA, int m, CUdeviceptr alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, cusparseSolveAnalysisInfo info, CUdeviceptr x, CUdeviceptr y);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDcsrsv_solve_v2(cusparseContext handle, cusparseOperation transA, int m, CUdeviceptr alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, cusparseSolveAnalysisInfo info, CUdeviceptr x, CUdeviceptr y);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCcsrsv_solve_v2(cusparseContext handle, cusparseOperation transA, int m, CUdeviceptr alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, cusparseSolveAnalysisInfo info, CUdeviceptr x, CUdeviceptr y);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZcsrsv_solve_v2(cusparseContext handle, cusparseOperation transA, int m, CUdeviceptr alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, cusparseSolveAnalysisInfo info, CUdeviceptr x, CUdeviceptr y);
		#endregion

		/* Description: Solution of triangular linear system op(A) * y = alpha * x, 
		   where A is a sparse matrix in HYB storage format, x and y are dense vectors. */
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseShybsv_analysis(cusparseContext handle, cusparseOperation transA, cusparseMatDescr descrA, cusparseHybMat hybA, cusparseSolveAnalysisInfo info);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDhybsv_analysis(cusparseContext handle, cusparseOperation transA, cusparseMatDescr descrA, cusparseHybMat hybA, cusparseSolveAnalysisInfo info);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseChybsv_analysis(cusparseContext handle, cusparseOperation transA, cusparseMatDescr descrA, cusparseHybMat hybA, cusparseSolveAnalysisInfo info);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZhybsv_analysis(cusparseContext handle, cusparseOperation transA, cusparseMatDescr descrA, cusparseHybMat hybA, cusparseSolveAnalysisInfo info);

		#region ref host
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseShybsv_solve(cusparseContext handle, cusparseOperation trans, ref float alpha, cusparseMatDescr descra, cusparseHybMat hybA, cusparseSolveAnalysisInfo info, CUdeviceptr x, CUdeviceptr y);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseChybsv_solve(cusparseContext handle, cusparseOperation trans, ref cuFloatComplex alpha, cusparseMatDescr descra, cusparseHybMat hybA, cusparseSolveAnalysisInfo info, CUdeviceptr x, CUdeviceptr y);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDhybsv_solve(cusparseContext handle, cusparseOperation trans, ref double alpha, cusparseMatDescr descra, cusparseHybMat hybA, cusparseSolveAnalysisInfo info, CUdeviceptr x, CUdeviceptr y);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZhybsv_solve(cusparseContext handle, cusparseOperation trans, ref cuDoubleComplex alpha, cusparseMatDescr descra, cusparseHybMat hybA, cusparseSolveAnalysisInfo info, CUdeviceptr x, CUdeviceptr y);
		#endregion
		#region ref device
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseShybsv_solve(cusparseContext handle, cusparseOperation trans, CUdeviceptr alpha, cusparseMatDescr descra, cusparseHybMat hybA, cusparseSolveAnalysisInfo info, CUdeviceptr x, CUdeviceptr y);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseChybsv_solve(cusparseContext handle, cusparseOperation trans, CUdeviceptr alpha, cusparseMatDescr descra, cusparseHybMat hybA, cusparseSolveAnalysisInfo info, CUdeviceptr x, CUdeviceptr y);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDhybsv_solve(cusparseContext handle, cusparseOperation trans, CUdeviceptr alpha, cusparseMatDescr descra, cusparseHybMat hybA, cusparseSolveAnalysisInfo info, CUdeviceptr x, CUdeviceptr y);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZhybsv_solve(cusparseContext handle, cusparseOperation trans, CUdeviceptr alpha, cusparseMatDescr descra, cusparseHybMat hybA, cusparseSolveAnalysisInfo info, CUdeviceptr x, CUdeviceptr y);
		#endregion

		#endregion

		#region Sparse Level 3 routines

		/* Description: Matrix-matrix multiplication C = alpha * op(A) * B  + beta * C, 
   where A is a sparse matrix, B and C are dense and usually tall matrices. */
		#region ref host
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseScsrmm_v2(cusparseContext handle, cusparseOperation transA, int m, int n, int k, int nnz, ref float alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, CUdeviceptr B, int ldb, ref float beta, CUdeviceptr C, int ldc);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDcsrmm_v2(cusparseContext handle, cusparseOperation transA, int m, int n, int k, int nnz, ref double alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, CUdeviceptr B, int ldb, ref double beta, CUdeviceptr C, int ldc);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCcsrmm_v2(cusparseContext handle, cusparseOperation transA, int m, int n, int k, int nnz, ref cuFloatComplex alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, CUdeviceptr B, int ldb, ref cuFloatComplex beta, CUdeviceptr C, int ldc);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZcsrmm_v2(cusparseContext handle, cusparseOperation transA, int m, int n, int k, int nnz, ref cuDoubleComplex alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, CUdeviceptr B, int ldb, ref cuDoubleComplex beta, CUdeviceptr C, int ldc);
		#endregion
		#region ref device
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseScsrmm_v2(cusparseContext handle, cusparseOperation transA, int m, int n, int k, int nnz, CUdeviceptr alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, CUdeviceptr B, int ldb, CUdeviceptr beta, CUdeviceptr C, int ldc);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDcsrmm_v2(cusparseContext handle, cusparseOperation transA, int m, int n, int k, int nnz, CUdeviceptr alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, CUdeviceptr B, int ldb, CUdeviceptr beta, CUdeviceptr C, int ldc);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCcsrmm_v2(cusparseContext handle, cusparseOperation transA, int m, int n, int k, int nnz, CUdeviceptr alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, CUdeviceptr B, int ldb, CUdeviceptr beta, CUdeviceptr C, int ldc);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZcsrmm_v2(cusparseContext handle, cusparseOperation transA, int m, int n, int k, int nnz, CUdeviceptr alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, CUdeviceptr B, int ldb, CUdeviceptr beta, CUdeviceptr C, int ldc);
		#endregion

		#region ref host
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseScsrmm2(cusparseContext handle, cusparseOperation transa, cusparseOperation transb, int m, int n, int k, int nnz,
                                            ref float alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, CUdeviceptr B, int ldb, ref float beta, CUdeviceptr C, int ldc);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDcsrmm2(cusparseContext handle, cusparseOperation transa, cusparseOperation transb, int m, int n, int k, int nnz,
                                            ref double alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, CUdeviceptr B, int ldb, ref double beta, CUdeviceptr C, int ldc);
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCcsrmm2(cusparseContext handle, cusparseOperation transa, cusparseOperation transb, int m, int n, int k, int nnz,
                                            ref cuFloatComplex alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, CUdeviceptr B, int ldb, ref cuFloatComplex beta, CUdeviceptr C, int ldc);
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZcsrmm2(cusparseContext handle, cusparseOperation transa, cusparseOperation transb, int m, int n, int k, int nnz,
											ref cuDoubleComplex alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, CUdeviceptr B, int ldb, ref cuDoubleComplex beta, CUdeviceptr C, int ldc);
		#endregion

		#region ref device

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseScsrmm2(cusparseContext handle, cusparseOperation transa, cusparseOperation transb, int m, int n, int k, int nnz,
											CUdeviceptr alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, CUdeviceptr B, int ldb, CUdeviceptr beta, CUdeviceptr C, int ldc);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDcsrmm2(cusparseContext handle, cusparseOperation transa, cusparseOperation transb, int m, int n, int k, int nnz,
											CUdeviceptr alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, CUdeviceptr B, int ldb, CUdeviceptr beta, CUdeviceptr C, int ldc);
		/// <summary/> 
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCcsrmm2(cusparseContext handle, cusparseOperation transa, cusparseOperation transb, int m, int n, int k, int nnz,
											CUdeviceptr alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, CUdeviceptr B, int ldb, CUdeviceptr beta, CUdeviceptr C, int ldc);
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZcsrmm2(cusparseContext handle, cusparseOperation transa, cusparseOperation transb, int m, int n, int k, int nnz,
											CUdeviceptr alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, CUdeviceptr B, int ldb, CUdeviceptr beta, CUdeviceptr C, int ldc);

		#endregion

		/* Description: Solution of triangular linear system op(A) * Y = alpha * X, 
		   with multiple right-hand-sides, where A is a sparse matrix in CSR storage 
		   format, X and Y are dense and usually tall matrices. */
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseScsrsm_analysis(cusparseContext handle, cusparseOperation transA, int m, int nnz, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, cusparseSolveAnalysisInfo info);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDcsrsm_analysis(cusparseContext handle, cusparseOperation transA, int m, int nnz, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, cusparseSolveAnalysisInfo info);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCcsrsm_analysis(cusparseContext handle, cusparseOperation transA, int m, int nnz, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, cusparseSolveAnalysisInfo info);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZcsrsm_analysis(cusparseContext handle, cusparseOperation transA, int m, int nnz, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, cusparseSolveAnalysisInfo info);


		#region ref host
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseScsrsm_solve(cusparseContext handle, cusparseOperation transA, int m, int n, ref float alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, cusparseSolveAnalysisInfo info, CUdeviceptr x, int ldx, CUdeviceptr y, int ldy);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDcsrsm_solve(cusparseContext handle, cusparseOperation transA, int m, int n, ref double alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, cusparseSolveAnalysisInfo info, CUdeviceptr x, int ldx, CUdeviceptr y, int ldy);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCcsrsm_solve(cusparseContext handle, cusparseOperation transA, int m, int n, ref cuFloatComplex alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, cusparseSolveAnalysisInfo info, CUdeviceptr x, int ldx, CUdeviceptr y, int ldy);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZcsrsm_solve(cusparseContext handle, cusparseOperation transA, int m, int n, ref cuDoubleComplex alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, cusparseSolveAnalysisInfo info, CUdeviceptr x, int ldx, CUdeviceptr y, int ldy);
		#endregion
		#region ref device
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseScsrsm_solve(cusparseContext handle, cusparseOperation transA, int m, int n, CUdeviceptr alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, cusparseSolveAnalysisInfo info, CUdeviceptr x, int ldx, CUdeviceptr y, int ldy);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDcsrsm_solve(cusparseContext handle, cusparseOperation transA, int m, int n, CUdeviceptr alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, cusparseSolveAnalysisInfo info, CUdeviceptr x, int ldx, CUdeviceptr y, int ldy);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCcsrsm_solve(cusparseContext handle, cusparseOperation transA, int m, int n, CUdeviceptr alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, cusparseSolveAnalysisInfo info, CUdeviceptr x, int ldx, CUdeviceptr y, int ldy);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZcsrsm_solve(cusparseContext handle, cusparseOperation transA, int m, int n, CUdeviceptr alpha, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, cusparseSolveAnalysisInfo info, CUdeviceptr x, int ldx, CUdeviceptr y, int ldy);
		#endregion

		/* Description: Compute the incomplete-LU factorization with 0 fill-in (ILU0)
   based on the information in the opaque structure info that was obtained 
   from the analysis phase (csrsv_analysis). */

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseScsrilu0(cusparseContext handle, 
                                              cusparseOperation trans, 
                                              int m, 
                                              cusparseMatDescr descrA, 
											  CUdeviceptr csrValM,
											  /* matrix A values are updated inplace 
                                                 to be the preconditioner M values */                                              
                                              CUdeviceptr csrRowPtrA, 
                                              CUdeviceptr csrColIndA,
                                              cusparseSolveAnalysisInfo info);


		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDcsrilu0(cusparseContext handle, 
                                              cusparseOperation trans, 
                                              int m, 
                                              cusparseMatDescr descrA,
											  CUdeviceptr csrValM,
											/* matrix A values are updated inplace 
											   to be the preconditioner M values */ 
											  CUdeviceptr csrRowPtrA,
											  CUdeviceptr csrColIndA,
                                              cusparseSolveAnalysisInfo info);


		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCcsrilu0(cusparseContext handle, 
                                              cusparseOperation trans, 
                                              int m, 
                                              cusparseMatDescr descrA,
											  CUdeviceptr csrValM, 
											/* matrix A values are updated inplace 
											   to be the preconditioner M values */ 
											  CUdeviceptr csrRowPtrA,
											  CUdeviceptr csrColIndA,
                                              cusparseSolveAnalysisInfo info);


		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZcsrilu0(cusparseContext handle, 
                                              cusparseOperation trans, 
                                              int m, 
                                              cusparseMatDescr descrA,
											  CUdeviceptr csrValM, 
											/* matrix A values are updated inplace 
											   to be the preconditioner M values */ 
											  CUdeviceptr csrRowPtrA,
											  CUdeviceptr csrColIndA,
                                              cusparseSolveAnalysisInfo info);

/* Description: Compute the incomplete-Cholesky factorization with 0 fill-in (IC0)
   based on the information in the opaque structure info that was obtained 
   from the analysis phase (csrsv_analysis). */

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseScsric0(cusparseContext handle, 
                                              cusparseOperation trans, 
                                              int m, 
                                              cusparseMatDescr descrA,
											  CUdeviceptr csrValA_ValM,
											/* matrix A values are updated inplace 
											   to be the preconditioner M values */ 
											  CUdeviceptr csrRowPtrA,
											  CUdeviceptr csrColIndA,
                                              cusparseSolveAnalysisInfo info);


		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDcsric0(cusparseContext handle, 
                                              cusparseOperation trans, 
                                              int m,
											  cusparseMatDescr descrA,
											  CUdeviceptr csrValA_ValM,
											/* matrix A values are updated inplace 
											   to be the preconditioner M values */
											  CUdeviceptr csrRowPtrA,
											  CUdeviceptr csrColIndA,
                                              cusparseSolveAnalysisInfo info);


		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCcsric0(cusparseContext handle, 
                                              cusparseOperation trans, 
                                              int m,
											  cusparseMatDescr descrA,
											  CUdeviceptr csrValA_ValM,
											/* matrix A values are updated inplace 
											   to be the preconditioner M values */
											  CUdeviceptr csrRowPtrA,
											  CUdeviceptr csrColIndA,
                                              cusparseSolveAnalysisInfo info);


		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZcsric0(cusparseContext handle, 
                                              cusparseOperation trans, 
                                              int m,
											  cusparseMatDescr descrA,
											  CUdeviceptr csrValA_ValM,
											/* matrix A values are updated inplace 
											   to be the preconditioner M values */
											  CUdeviceptr csrRowPtrA,
											  CUdeviceptr csrColIndA,
                                              cusparseSolveAnalysisInfo info);




		/* Description: Solution of tridiagonal linear system A * B = B, 
		   with multiple right-hand-sides. The coefficient matrix A is 
		   composed of lower (dl), main (d) and upper (du) diagonals, and 
		   the right-hand-sides B are overwritten with the solution. */
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseSgtsv(cusparseContext handle, int m, int n, CUdeviceptr dl, CUdeviceptr d, CUdeviceptr du, CUdeviceptr B, int ldb);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDgtsv(cusparseContext handle, int m, int n, CUdeviceptr dl, CUdeviceptr d, CUdeviceptr du, CUdeviceptr B, int ldb);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCgtsv(cusparseContext handle, int m, int n, CUdeviceptr dl, CUdeviceptr d, CUdeviceptr du, CUdeviceptr B, int ldb);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZgtsv(cusparseContext handle, int m, int n, CUdeviceptr dl, CUdeviceptr d, CUdeviceptr du, CUdeviceptr B, int ldb);


		/* Description: Solution of tridiagonal linear system A * B = B, 
   with multiple right-hand-sides. The coefficient matrix A is 
   composed of lower (dl), main (d) and upper (du) diagonals, and 
   the right-hand-sides B are overwritten with the solution. 
   These routines do not use pivoting, using a combination of PCR and CR algorithm */                               
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseSgtsv_nopivot(cusparseContext handle, int m, int n, CUdeviceptr dl, CUdeviceptr d, CUdeviceptr du, CUdeviceptr B, int ldb);
                                 
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDgtsv_nopivot(cusparseContext handle, int m, int n, CUdeviceptr dl, CUdeviceptr d, CUdeviceptr du, CUdeviceptr B, int ldb);
                                                                 
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCgtsv_nopivot(cusparseContext handle, int m, int n, CUdeviceptr dl, CUdeviceptr d, CUdeviceptr du, CUdeviceptr B, int ldb);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZgtsv_nopivot(cusparseContext handle, int m, int n, CUdeviceptr dl, CUdeviceptr d, CUdeviceptr du, CUdeviceptr B, int ldb);                               
                                  


		/* Description: Solution of a set of tridiagonal linear systems 
		   A * x = x, each with a single right-hand-side. The coefficient 
		   matrices A are composed of lower (dl), main (d) and upper (du) 
		   diagonals and stored separated by a batchStride, while the 
		   right-hand-sides x are also separated by a batchStride. */
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseSgtsvStridedBatch(cusparseContext handle, int m, CUdeviceptr dl, CUdeviceptr d, CUdeviceptr du, CUdeviceptr x, int batchCount, int batchStride);


		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDgtsvStridedBatch(cusparseContext handle, int m, CUdeviceptr dl, CUdeviceptr d, CUdeviceptr du, CUdeviceptr x, int batchCount, int batchStride);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCgtsvStridedBatch(cusparseContext handle, int m, CUdeviceptr dl, CUdeviceptr d, CUdeviceptr du, CUdeviceptr x, int batchCount, int batchStride);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZgtsvStridedBatch(cusparseContext handle, int m, CUdeviceptr dl, CUdeviceptr d, CUdeviceptr du, CUdeviceptr x, int batchCount, int batchStride);


		/* --- Sparse Format Conversion --- */

		/* Description: This routine finds the total number of non-zero elements and 
		   the number of non-zero elements per row or column in the dense matrix A. */
		#region ref host
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseSnnz(cusparseContext handle, cusparseDirection dirA, int m, int n, cusparseMatDescr descrA, CUdeviceptr A, int lda, CUdeviceptr nnzPerRowCol, ref int nnzTotalDevHostPtr);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDnnz(cusparseContext handle, cusparseDirection dirA, int m, int n, cusparseMatDescr descrA, CUdeviceptr A, int lda, CUdeviceptr nnzPerRowCol, ref int nnzTotalDevHostPtr);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCnnz(cusparseContext handle, cusparseDirection dirA, int m, int n, cusparseMatDescr descrA, CUdeviceptr A, int lda, CUdeviceptr nnzPerRowCol, ref int nnzTotalDevHostPtr);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZnnz(cusparseContext handle, cusparseDirection dirA, int m, int n, cusparseMatDescr descrA, CUdeviceptr A, int lda, CUdeviceptr nnzPerRowCol, ref int nnzTotalDevHostPtr);
		#endregion
		#region ref device
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseSnnz(cusparseContext handle, cusparseDirection dirA, int m, int n, cusparseMatDescr descrA, CUdeviceptr A, int lda, CUdeviceptr nnzPerRowCol, CUdeviceptr nnzTotalDevHostPtr);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDnnz(cusparseContext handle, cusparseDirection dirA, int m, int n, cusparseMatDescr descrA, CUdeviceptr A, int lda, CUdeviceptr nnzPerRowCol, CUdeviceptr nnzTotalDevHostPtr);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCnnz(cusparseContext handle, cusparseDirection dirA, int m, int n, cusparseMatDescr descrA, CUdeviceptr A, int lda, CUdeviceptr nnzPerRowCol, CUdeviceptr nnzTotalDevHostPtr);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZnnz(cusparseContext handle, cusparseDirection dirA, int m, int n, cusparseMatDescr descrA, CUdeviceptr A, int lda, CUdeviceptr nnzPerRowCol, CUdeviceptr nnzTotalDevHostPtr);
		#endregion

		/* Description: This routine converts a dense matrix to a sparse matrix 
		   in the CSR storage format, using the information computed by the 
		   nnz routine. */
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseSdense2csr(cusparseContext handle, int m, int n, cusparseMatDescr descrA, CUdeviceptr A, int lda, CUdeviceptr nnzPerRow, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDdense2csr(cusparseContext handle, int m, int n, cusparseMatDescr descrA, CUdeviceptr A, int lda, CUdeviceptr nnzPerRow, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCdense2csr(cusparseContext handle, int m, int n, cusparseMatDescr descrA, CUdeviceptr A, int lda, CUdeviceptr nnzPerRow, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZdense2csr(cusparseContext handle, int m, int n, cusparseMatDescr descrA, CUdeviceptr A, int lda, CUdeviceptr nnzPerRow, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA);

		/* Description: This routine converts a sparse matrix in CSR storage format
		   to a dense matrix. */
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseScsr2dense(cusparseContext handle, int m, int n, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, CUdeviceptr A, int lda);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDcsr2dense(cusparseContext handle, int m, int n, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, CUdeviceptr A, int lda);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCcsr2dense(cusparseContext handle, int m, int n, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, CUdeviceptr A, int lda);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZcsr2dense(cusparseContext handle, int m, int n, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, CUdeviceptr A, int lda);

		/* Description: This routine converts a dense matrix to a sparse matrix 
		   in the CSC storage format, using the information computed by the 
		   nnz routine. */
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseSdense2csc(cusparseContext handle, int m, int n, cusparseMatDescr descrA, CUdeviceptr A, int lda, CUdeviceptr nnzPerCol, CUdeviceptr cscValA, CUdeviceptr cscRowIndA, CUdeviceptr cscColPtrA);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDdense2csc(cusparseContext handle, int m, int n, cusparseMatDescr descrA, CUdeviceptr A, int lda, CUdeviceptr nnzPerCol, CUdeviceptr cscValA, CUdeviceptr cscRowIndA, CUdeviceptr cscColPtrA);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCdense2csc(cusparseContext handle, int m, int n, cusparseMatDescr descrA, CUdeviceptr A, int lda, CUdeviceptr nnzPerCol, CUdeviceptr cscValA, CUdeviceptr cscRowIndA, CUdeviceptr cscColPtrA);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZdense2csc(cusparseContext handle, int m, int n, cusparseMatDescr descrA, CUdeviceptr A, int lda, CUdeviceptr nnzPerCol, CUdeviceptr cscValA, CUdeviceptr cscRowIndA, CUdeviceptr cscColPtrA);

		/* Description: This routine converts a sparse matrix in CSC storage format
		   to a dense matrix. */
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseScsc2dense(cusparseContext handle, int m, int n, cusparseMatDescr descrA, CUdeviceptr cscValA, CUdeviceptr cscRowIndA, CUdeviceptr cscColPtrA, CUdeviceptr A, int lda);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDcsc2dense(cusparseContext handle, int m, int n, cusparseMatDescr descrA, CUdeviceptr cscValA, CUdeviceptr cscRowIndA, CUdeviceptr cscColPtrA, CUdeviceptr A, int lda);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCcsc2dense(cusparseContext handle, int m, int n, cusparseMatDescr descrA, CUdeviceptr cscValA, CUdeviceptr cscRowIndA, CUdeviceptr cscColPtrA, CUdeviceptr A, int lda);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZcsc2dense(cusparseContext handle, int m, int n, cusparseMatDescr descrA, CUdeviceptr cscValA, CUdeviceptr cscRowIndA, CUdeviceptr cscColPtrA, CUdeviceptr A, int lda);

		/* Description: This routine compresses the indecis of rows or columns.
		   It can be interpreted as a conversion from COO to CSR sparse storage
		   format. */
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseXcoo2csr(cusparseContext handle, CUdeviceptr cooRowInd, int nnz, int m, CUdeviceptr csrRowPtr, cusparseIndexBase idxBase);

		/* Description: This routine uncompresses the indecis of rows or columns.
		   It can be interpreted as a conversion from CSR to COO sparse storage
		   format. */
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseXcsr2coo(cusparseContext handle, CUdeviceptr csrRowPtr, int nnz, int m, CUdeviceptr cooRowInd, cusparseIndexBase idxBase);

		/* Description: This routine converts a matrix from CSR to CSC sparse 
		   storage format. The resulting matrix can be re-interpreted as a 
		   transpose of the original matrix in CSR storage format. */
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseScsr2csc_v2(cusparseContext handle, int m, int n, int nnz, CUdeviceptr csrVal, CUdeviceptr csrRowPtr, CUdeviceptr csrColInd, CUdeviceptr cscVal, CUdeviceptr cscRowInd, CUdeviceptr cscColPtr, cusparseAction copyValues, cusparseIndexBase idxBase);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDcsr2csc_v2(cusparseContext handle, int m, int n, int nnz, CUdeviceptr csrVal, CUdeviceptr csrRowPtr, CUdeviceptr csrColInd, CUdeviceptr cscVal, CUdeviceptr cscRowInd, CUdeviceptr cscColPtr, cusparseAction copyValues, cusparseIndexBase idxBase);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCcsr2csc_v2(cusparseContext handle, int m, int n, int nnz, CUdeviceptr csrVal, CUdeviceptr csrRowPtr, CUdeviceptr csrColInd, CUdeviceptr cscVal, CUdeviceptr cscRowInd, CUdeviceptr cscColPtr, cusparseAction copyValues, cusparseIndexBase idxBase);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZcsr2csc_v2(cusparseContext handle, int m, int n, int nnz, CUdeviceptr csrVal, CUdeviceptr csrRowPtr, CUdeviceptr csrColInd, CUdeviceptr cscVal, CUdeviceptr cscRowInd, CUdeviceptr cscColPtr, cusparseAction copyValues, cusparseIndexBase idxBase);

		/* Description: This routine converts a dense matrix to a sparse matrix 
		   in HYB storage format. */
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseSdense2hyb(cusparseContext handle, int m, int n, cusparseMatDescr descrA, CUdeviceptr A, int lda, CUdeviceptr nnzPerRow, cusparseHybMat hybA, int userEllWidth, cusparseHybPartition partitionType);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDdense2hyb(cusparseContext handle, int m, int n, cusparseMatDescr descrA, CUdeviceptr A, int lda, CUdeviceptr nnzPerRow, cusparseHybMat hybA, int userEllWidth, cusparseHybPartition partitionType);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCdense2hyb(cusparseContext handle, int m, int n, cusparseMatDescr descrA, CUdeviceptr A, int lda, CUdeviceptr nnzPerRow, cusparseHybMat hybA, int userEllWidth, cusparseHybPartition partitionType);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZdense2hyb(cusparseContext handle, int m, int n, cusparseMatDescr descrA, CUdeviceptr A, int lda, CUdeviceptr nnzPerRow, cusparseHybMat hybA, int userEllWidth, cusparseHybPartition partitionType);

		/* Description: This routine converts a sparse matrix in HYB storage format
		   to a dense matrix. */
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseShyb2dense(cusparseContext handle, cusparseMatDescr descrA, cusparseHybMat hybA, CUdeviceptr A, int lda);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDhyb2dense(cusparseContext handle, cusparseMatDescr descrA, cusparseHybMat hybA, CUdeviceptr A, int lda);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseChyb2dense(cusparseContext handle, cusparseMatDescr descrA, cusparseHybMat hybA, CUdeviceptr A, int lda);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZhyb2dense(cusparseContext handle, cusparseMatDescr descrA, cusparseHybMat hybA, CUdeviceptr A, int lda);

		/* Description: This routine converts a sparse matrix in CSR storage format
		   to a sparse matrix in HYB storage format. */
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseScsr2hyb(cusparseContext handle, int m, int n, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, cusparseHybMat hybA, int userEllWidth, cusparseHybPartition partitionType);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDcsr2hyb(cusparseContext handle, int m, int n, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, cusparseHybMat hybA, int userEllWidth, cusparseHybPartition partitionType);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCcsr2hyb(cusparseContext handle, int m, int n, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, cusparseHybMat hybA, int userEllWidth, cusparseHybPartition partitionType);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZcsr2hyb(cusparseContext handle, int m, int n, cusparseMatDescr descrA, CUdeviceptr csrValA, CUdeviceptr csrRowPtrA, CUdeviceptr csrColIndA, cusparseHybMat hybA, int userEllWidth, cusparseHybPartition partitionType);



		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseXcsrgemmNnz(cusparseContext handle,
												  cusparseOperation transA,
												  cusparseOperation transB,
												  int m,
												  int n,
												  int k,
												  cusparseMatDescr descrA,
												  int nnzA,
												  CUdeviceptr csrRowPtrA,
												  CUdeviceptr csrColIndA,
												  cusparseMatDescr descrB,
												  int nnzB,
												  CUdeviceptr csrRowPtrB,
												  CUdeviceptr csrColIndB,
												  cusparseMatDescr descrC,
												  CUdeviceptr csrRowPtrC,
												  ref int nnzTotalDevHostPtr);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseXcsrgemmNnz(cusparseContext handle,
												  cusparseOperation transA,
												  cusparseOperation transB,
												  int m,
												  int n,
												  int k,
												  cusparseMatDescr descrA,
												  int nnzA,
												  CUdeviceptr csrRowPtrA,
												  CUdeviceptr csrColIndA,
												  cusparseMatDescr descrB,
												  int nnzB,
												  CUdeviceptr csrRowPtrB,
												  CUdeviceptr csrColIndB,
												  cusparseMatDescr descrC,
												  CUdeviceptr csrRowPtrC,
												  CUdeviceptr nnzTotalDevHostPtr);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseScsrgemm(cusparseContext handle,
											  cusparseOperation transA,
											  cusparseOperation transB,
											  int m,
											  int n,
											  int k,
											  cusparseMatDescr descrA,
											  int nnzA,
											  CUdeviceptr csrValA,
											  CUdeviceptr csrRowPtrA,
											  CUdeviceptr csrColIndA,
											  cusparseMatDescr descrB,
											  int nnzB,
											  CUdeviceptr csrValB,
											  CUdeviceptr csrRowPtrB,
											  CUdeviceptr csrColIndB,
											  cusparseMatDescr descrC,
											  CUdeviceptr csrValC,
											  CUdeviceptr csrRowPtrC,
											  CUdeviceptr csrColIndC);



		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDcsrgemm(cusparseContext handle,
											  cusparseOperation transA,
											  cusparseOperation transB,
											  int m,
											  int n,
											  int k,
											  cusparseMatDescr descrA,
											  int nnzA,
											  CUdeviceptr csrValA,
											  CUdeviceptr csrRowPtrA,
											  CUdeviceptr csrColIndA,
											  cusparseMatDescr descrB,
											  int nnzB,
											  CUdeviceptr csrValB,
											  CUdeviceptr csrRowPtrB,
											  CUdeviceptr csrColIndB,
											  cusparseMatDescr descrC,
											  CUdeviceptr csrValC,
											  CUdeviceptr csrRowPtrC,
											  CUdeviceptr csrColIndC);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCcsrgemm(cusparseContext handle,
											  cusparseOperation transA,
											  cusparseOperation transB,
											  int m,
											  int n,
											  int k,
											  cusparseMatDescr descrA,
											  int nnzA,
											  CUdeviceptr csrValA,
											  CUdeviceptr csrRowPtrA,
											  CUdeviceptr csrColIndA,
											  cusparseMatDescr descrB,
											  int nnzB,
											  CUdeviceptr csrValB,
											  CUdeviceptr csrRowPtrB,
											  CUdeviceptr csrColIndB,
											  cusparseMatDescr descrC,
											  CUdeviceptr csrValC,
											  CUdeviceptr csrRowPtrC,
											  CUdeviceptr csrColIndC);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZcsrgemm(cusparseContext handle,
											  cusparseOperation transA,
											  cusparseOperation transB,
											  int m,
											  int n,
											  int k,
											  cusparseMatDescr descrA,
											  int nnzA,
											  CUdeviceptr csrValA,
											  CUdeviceptr csrRowPtrA,
											  CUdeviceptr csrColIndA,
											  cusparseMatDescr descrB,
											  int nnzB,
											  CUdeviceptr csrValB,
											  CUdeviceptr csrRowPtrB,
											  CUdeviceptr csrColIndB,
											  cusparseMatDescr descrC,
											  CUdeviceptr csrValC,
											  CUdeviceptr csrRowPtrC,
											  CUdeviceptr csrColIndC);


		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseXcsrgeamNnz(cusparseContext handle,
											   int m,
											   int n,
											   cusparseMatDescr descrA,
											   int nnzA,
											   CUdeviceptr csrRowPtrA,
											   CUdeviceptr csrColIndA,
											   cusparseMatDescr descrB,
											   int nnzB,
											   CUdeviceptr csrRowPtrB,
											   CUdeviceptr csrColIndB,
											   cusparseMatDescr descrC,
											   CUdeviceptr csrRowPtrC,
											   ref int nnzTotalDevHostPtr);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseXcsrgeamNnz(cusparseContext handle,
											   int m,
											   int n,
											   cusparseMatDescr descrA,
											   int nnzA,
											   CUdeviceptr csrRowPtrA,
											   CUdeviceptr csrColIndA,
											   cusparseMatDescr descrB,
											   int nnzB,
											   CUdeviceptr csrRowPtrB,
											   CUdeviceptr csrColIndB,
											   cusparseMatDescr descrC,
											   CUdeviceptr csrRowPtrC,
											   CUdeviceptr nnzTotalDevHostPtr);

		#region host
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseScsrgeam(cusparseContext handle,
											int m,
											int n,
											ref float alpha,
											cusparseMatDescr descrA,
											int nnzA,
											CUdeviceptr csrValA,
											CUdeviceptr csrRowPtrA,
											CUdeviceptr csrColIndA,
											ref float beta,
											cusparseMatDescr descrB,
											int nnzB,
											CUdeviceptr csrValB,
											CUdeviceptr csrRowPtrB,
											CUdeviceptr csrColIndB,
											cusparseMatDescr descrC,
											CUdeviceptr csrValC,
											CUdeviceptr csrRowPtrC,
											CUdeviceptr csrColIndC);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDcsrgeam(cusparseContext handle,
											int m,
											int n,
											ref double alpha,
											cusparseMatDescr descrA,
											int nnzA,
											CUdeviceptr csrValA,
											CUdeviceptr csrRowPtrA,
											CUdeviceptr csrColIndA,
											ref double beta,
											cusparseMatDescr descrB,
											int nnzB,
											CUdeviceptr csrValB,
											CUdeviceptr csrRowPtrB,
											CUdeviceptr csrColIndB,
											cusparseMatDescr descrC,
											CUdeviceptr csrValC,
											CUdeviceptr csrRowPtrC,
											CUdeviceptr csrColIndC);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCcsrgeam(cusparseContext handle,
											int m,
											int n,
											ref cuFloatComplex alpha,
											cusparseMatDescr descrA,
											int nnzA,
											CUdeviceptr csrValA,
											CUdeviceptr csrRowPtrA,
											CUdeviceptr csrColIndA,
											ref cuFloatComplex beta,
											cusparseMatDescr descrB,
											int nnzB,
											CUdeviceptr csrValB,
											CUdeviceptr csrRowPtrB,
											CUdeviceptr csrColIndB,
											cusparseMatDescr descrC,
											CUdeviceptr csrValC,
											CUdeviceptr csrRowPtrC,
											CUdeviceptr csrColIndC);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZcsrgeam(cusparseContext handle,
											int m,
											int n,
											ref cuDoubleComplex alpha,
											cusparseMatDescr descrA,
											int nnzA,
											CUdeviceptr csrValA,
											CUdeviceptr csrRowPtrA,
											CUdeviceptr csrColIndA,
											ref cuDoubleComplex beta,
											cusparseMatDescr descrB,
											int nnzB,
											CUdeviceptr csrValB,
											CUdeviceptr csrRowPtrB,
											CUdeviceptr csrColIndB,
											cusparseMatDescr descrC,
											CUdeviceptr csrValC,
											CUdeviceptr csrRowPtrC,
											CUdeviceptr csrColIndC);
		#endregion
		#region device
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseScsrgeam(cusparseContext handle,
											int m,
											int n,
											CUdeviceptr alpha,
											cusparseMatDescr descrA,
											int nnzA,
											CUdeviceptr csrValA,
											CUdeviceptr csrRowPtrA,
											CUdeviceptr csrColIndA,
											CUdeviceptr beta,
											cusparseMatDescr descrB,
											int nnzB,
											CUdeviceptr csrValB,
											CUdeviceptr csrRowPtrB,
											CUdeviceptr csrColIndB,
											cusparseMatDescr descrC,
											CUdeviceptr csrValC,
											CUdeviceptr csrRowPtrC,
											CUdeviceptr csrColIndC);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDcsrgeam(cusparseContext handle,
											int m,
											int n,
											CUdeviceptr alpha,
											cusparseMatDescr descrA,
											int nnzA,
											CUdeviceptr csrValA,
											CUdeviceptr csrRowPtrA,
											CUdeviceptr csrColIndA,
											CUdeviceptr beta,
											cusparseMatDescr descrB,
											int nnzB,
											CUdeviceptr csrValB,
											CUdeviceptr csrRowPtrB,
											CUdeviceptr csrColIndB,
											cusparseMatDescr descrC,
											CUdeviceptr csrValC,
											CUdeviceptr csrRowPtrC,
											CUdeviceptr csrColIndC);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCcsrgeam(cusparseContext handle,
											int m,
											int n,
											CUdeviceptr alpha,
											cusparseMatDescr descrA,
											int nnzA,
											CUdeviceptr csrValA,
											CUdeviceptr csrRowPtrA,
											CUdeviceptr csrColIndA,
											CUdeviceptr beta,
											cusparseMatDescr descrB,
											int nnzB,
											CUdeviceptr csrValB,
											CUdeviceptr csrRowPtrB,
											CUdeviceptr csrColIndB,
											cusparseMatDescr descrC,
											CUdeviceptr csrValC,
											CUdeviceptr csrRowPtrC,
											CUdeviceptr csrColIndC);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZcsrgeam(cusparseContext handle,
											int m,
											int n,
											CUdeviceptr alpha,
											cusparseMatDescr descrA,
											int nnzA,
											CUdeviceptr csrValA,
											CUdeviceptr csrRowPtrA,
											CUdeviceptr csrColIndA,
											CUdeviceptr beta,
											cusparseMatDescr descrB,
											int nnzB,
											CUdeviceptr csrValB,
											CUdeviceptr csrRowPtrB,
											CUdeviceptr csrColIndB,
											cusparseMatDescr descrC,
											CUdeviceptr csrValC,
											CUdeviceptr csrRowPtrC,
											CUdeviceptr csrColIndC);
		#endregion
		
		/* Description: This routine converts a sparse matrix in HYB storage format
		   to a sparse matrix in CSR storage format. */

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseShyb2csr(cusparseContext handle,
                                              cusparseMatDescr descrA,
                                              cusparseHybMat hybA,
                                              CUdeviceptr csrValA,
                                              CUdeviceptr csrRowPtrA,
                                              CUdeviceptr csrColIndA);


		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDhyb2csr(cusparseContext handle,
                                              cusparseMatDescr descrA,
                                              cusparseHybMat hybA,
                                              CUdeviceptr csrValA,
                                              CUdeviceptr csrRowPtrA,
                                              CUdeviceptr csrColIndA);              


		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseChyb2csr(cusparseContext handle,
                                              cusparseMatDescr descrA,
                                              cusparseHybMat hybA,
                                              CUdeviceptr csrValA,
                                              CUdeviceptr csrRowPtrA,
                                              CUdeviceptr csrColIndA);


		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZhyb2csr(cusparseContext handle,
                                              cusparseMatDescr descrA,
                                              cusparseHybMat hybA,
                                              CUdeviceptr csrValA,
                                              CUdeviceptr csrRowPtrA,
                                              CUdeviceptr csrColIndA);


		/* Description: This routine converts a sparse matrix in CSC storage format
   to a sparse matrix in HYB storage format. */
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseScsc2hyb(cusparseContext handle,
                                              int m,
                                              int n,
                                              cusparseMatDescr descrA,
                                              CUdeviceptr cscValA,
                                              CUdeviceptr cscRowIndA,
                                              CUdeviceptr cscColPtrA,
                                              cusparseHybMat hybA,
                                              int userEllWidth,
                                              cusparseHybPartition partitionType);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDcsc2hyb(cusparseContext handle,
                                              int m,
                                              int n,
                                              cusparseMatDescr descrA,
                                              CUdeviceptr cscValA,
                                              CUdeviceptr cscRowIndA,
                                              CUdeviceptr cscColPtrA,
                                              cusparseHybMat hybA,
                                              int userEllWidth,
                                              cusparseHybPartition partitionType);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCcsc2hyb(cusparseContext handle,
                                              int m,
                                              int n,
                                              cusparseMatDescr descrA,
                                              CUdeviceptr cscValA,
                                              CUdeviceptr cscRowIndA,
                                              CUdeviceptr cscColPtrA,
                                              cusparseHybMat hybA,
                                              int userEllWidth,
                                              cusparseHybPartition partitionType);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZcsc2hyb(cusparseContext handle,
                                              int m,
                                              int n,
                                              cusparseMatDescr descrA,
                                              CUdeviceptr cscValA,
                                              CUdeviceptr cscRowIndA,
                                              CUdeviceptr cscColPtrA,
                                              cusparseHybMat hybA,
                                              int userEllWidth,
                                              cusparseHybPartition partitionType);

/* Description: This routine converts a sparse matrix in HYB storage format
   to a sparse matrix in CSC storage format. */
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseShyb2csc(cusparseContext handle,
                                              cusparseMatDescr descrA,
                                              cusparseHybMat hybA,
                                              CUdeviceptr cscVal,
											  CUdeviceptr cscRowInd,
											  CUdeviceptr cscColPtr);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDhyb2csc(cusparseContext handle,
                                              cusparseMatDescr descrA,
                                              cusparseHybMat hybA,
											  CUdeviceptr cscVal,
											  CUdeviceptr cscRowInd,
											  CUdeviceptr cscColPtr);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseChyb2csc(cusparseContext handle,
                                              cusparseMatDescr descrA,
                                              cusparseHybMat hybA,
											  CUdeviceptr cscVal,
											  CUdeviceptr cscRowInd,
											  CUdeviceptr cscColPtr);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZhyb2csc(cusparseContext handle,
                                              cusparseMatDescr descrA,
                                              cusparseHybMat hybA,
											  CUdeviceptr cscVal,
											  CUdeviceptr cscRowInd,
											  CUdeviceptr cscColPtr);





		/* Description: This routine converts a sparse matrix in CSR storage format
		to a sparse matrix in BSR storage format. */
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseXcsr2bsrNnz(cusparseContext handle,
											cusparseDirection dirA,
											int m,
											int n,
											cusparseMatDescr descrA,
											CUdeviceptr csrRowPtrA,
											CUdeviceptr csrColIndA,
											int blockDim,
											cusparseMatDescr descrC,
											CUdeviceptr bsrRowPtrC,
											CUdeviceptr nnzTotalDevHostPtr);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseXcsr2bsrNnz(cusparseContext handle,
											cusparseDirection dirA,
											int m,
											int n,
											cusparseMatDescr descrA,
											CUdeviceptr csrRowPtrA,
											CUdeviceptr csrColIndA,
											int blockDim,
											cusparseMatDescr descrC,
											CUdeviceptr bsrRowPtrC,
											ref int nnzTotalDevHostPtr);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseScsr2bsr(cusparseContext handle,
											cusparseDirection dirA,
											int m,
											int n,
											cusparseMatDescr descrA,
											CUdeviceptr csrValA,
											CUdeviceptr csrRowPtrA,
											CUdeviceptr csrColIndA,
											int blockDim,
											cusparseMatDescr descrC,
											CUdeviceptr bsrValC,
											CUdeviceptr bsrRowPtrC,
											CUdeviceptr bsrColIndC);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDcsr2bsr(cusparseContext handle,
											cusparseDirection dirA,
											int m,
											int n,
											cusparseMatDescr descrA,
											CUdeviceptr csrValA,
											CUdeviceptr csrRowPtrA,
											CUdeviceptr csrColIndA,
											int blockDim,
											cusparseMatDescr descrC,
											CUdeviceptr bsrValC,
											CUdeviceptr bsrRowPtrC,
											CUdeviceptr bsrColIndC);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCcsr2bsr(cusparseContext handle,
											cusparseDirection dirA,
											int m,
											int n,
											cusparseMatDescr descrA,
											CUdeviceptr csrValA,
											CUdeviceptr csrRowPtrA,
											CUdeviceptr csrColIndA,
											int blockDim,
											cusparseMatDescr descrC,
											CUdeviceptr bsrValC,
											CUdeviceptr bsrRowPtrC,
											CUdeviceptr bsrColIndC);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZcsr2bsr(cusparseContext handle,
											cusparseDirection dirA,
											int m,
											int n,
											cusparseMatDescr descrA,
											CUdeviceptr csrValA,
											CUdeviceptr csrRowPtrA,
											CUdeviceptr csrColIndA,
											int blockDim,
											cusparseMatDescr descrC,
											CUdeviceptr bsrValC,
											CUdeviceptr bsrRowPtrC,
											CUdeviceptr bsrColIndC);


		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseSbsr2csr(cusparseContext handle,
											cusparseDirection dirA,
											int mb,
											int nb,
											cusparseMatDescr descrA,
											CUdeviceptr bsrValA,
											CUdeviceptr bsrRowPtrA,
											CUdeviceptr bsrColIndA,
											int blockDim,
											cusparseMatDescr descrC,
											CUdeviceptr csrValC,
											CUdeviceptr csrRowPtrC,
											CUdeviceptr csrColIndC);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDbsr2csr(cusparseContext handle,
											cusparseDirection dirA,
											int mb,
											int nb,
											cusparseMatDescr descrA,
											CUdeviceptr bsrValA,
											CUdeviceptr bsrRowPtrA,
											CUdeviceptr bsrColIndA,
											int blockDim,
											cusparseMatDescr descrC,
											CUdeviceptr csrValC,
											CUdeviceptr csrRowPtrC,
											CUdeviceptr csrColIndC);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCbsr2csr(cusparseContext handle,
											cusparseDirection dirA,
											int mb,
											int nb,
											cusparseMatDescr descrA,
											CUdeviceptr bsrValA,
											CUdeviceptr bsrRowPtrA,
											CUdeviceptr bsrColIndA,
											int blockDim,
											cusparseMatDescr descrC,
											CUdeviceptr csrValC,
											CUdeviceptr csrRowPtrC,
											CUdeviceptr csrColIndC);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZbsr2csr(cusparseContext handle,
											cusparseDirection dirA,
											int mb,
											int nb,
											cusparseMatDescr descrA,
											CUdeviceptr bsrValA,
											CUdeviceptr bsrRowPtrA,
											CUdeviceptr bsrColIndA,
											int blockDim,
											cusparseMatDescr descrC,
											CUdeviceptr csrValC,
											CUdeviceptr csrRowPtrC,
											CUdeviceptr csrColIndC);



		#region Removed in Cuda 5.5 production release

		///// <summary/>
		//[DllImport(CUSPARSE_API_DLL_NAME)]
		//public static extern cusparseStatus cusparseSgebsr2gebsc(cusparseContext handle,
		//                                      int mb,
		//                                      int nb,
		//                                      int nnzb,
		//                                      CUdeviceptr bsrVal,
		//                                      CUdeviceptr bsrRowPtr,
		//                                      CUdeviceptr bsrColInd,
		//                                      int rowBlockDim,
		//                                      int colBlockDim,
		//                                      CUdeviceptr bscVal,
		//                                      CUdeviceptr bscRowInd,
		//                                      CUdeviceptr bscColPtr,
		//                                      cusparseAction copyValues,
		//                                      cusparseIndexBase baseIdx);

		///// <summary/>
		//[DllImport(CUSPARSE_API_DLL_NAME)]
		//public static extern cusparseStatus cusparseDgebsr2gebsc(cusparseContext handle,
		//                                      int mb,
		//                                      int nb,
		//                                      int nnzb,
		//                                      CUdeviceptr bsrVal,
		//                                      CUdeviceptr bsrRowPtr,
		//                                      CUdeviceptr bsrColInd,
		//                                      int rowBlockDim,
		//                                      int colBlockDim,
		//                                      CUdeviceptr bscVal,
		//                                      CUdeviceptr bscRowInd,
		//                                      CUdeviceptr bscColPtr,
		//                                      cusparseAction copyValues,
		//                                      cusparseIndexBase baseIdx);

		///// <summary/>
		//[DllImport(CUSPARSE_API_DLL_NAME)]
		//public static extern cusparseStatus cusparseCgebsr2gebsc(cusparseContext handle,
		//                                      int mb,
		//                                      int nb,
		//                                      int nnzb,
		//                                      CUdeviceptr bsrVal,
		//                                      CUdeviceptr bsrRowPtr,
		//                                      CUdeviceptr bsrColInd,
		//                                      int rowBlockDim,
		//                                      int colBlockDim,
		//                                      CUdeviceptr bscVal,
		//                                      CUdeviceptr bscRowInd,
		//                                      CUdeviceptr bscColPtr,
		//                                      cusparseAction copyValues,
		//                                      cusparseIndexBase baseIdx);

		///// <summary/>
		//[DllImport(CUSPARSE_API_DLL_NAME)]
		//public static extern cusparseStatus cusparseZgebsr2gebsc(cusparseContext handle,
		//                                      int mb,
		//                                      int nb,
		//                                      int nnzb,
		//                                      CUdeviceptr bsrVal,
		//                                      CUdeviceptr bsrRowPtr,
		//                                      CUdeviceptr bsrColInd,
		//                                      int rowBlockDim,
		//                                      int colBlockDim,
		//                                      CUdeviceptr bscVal,
		//                                      CUdeviceptr bscRowInd,
		//                                      CUdeviceptr bscColPtr,
		//                                      cusparseAction copyValues,
		//                                      cusparseIndexBase baseIdx);

		///// <summary/>
		//[DllImport(CUSPARSE_API_DLL_NAME)]
		//public static extern cusparseStatus cusparseXgebsr2csr(cusparseContext handle,
		//                                      cusparseDirection dirA,
		//                                      int mb,
		//                                      int nb,
		//                                      cusparseMatDescr descrA,
		//                                      CUdeviceptr bsrRowPtrA,
		//                                      CUdeviceptr bsrColIndA,
		//                                      int   rowBlockDim,
		//                                      int   colBlockDim,
		//                                      cusparseMatDescr descrC,
		//                                      CUdeviceptr csrRowPtrC,
		//                                      CUdeviceptr csrColIndC );

		///// <summary/>
		//[DllImport(CUSPARSE_API_DLL_NAME)]
		//public static extern cusparseStatus cusparseSgebsr2csr(cusparseContext handle,
		//                                      cusparseDirection dirA,
		//                                      int mb,
		//                                      int nb,
		//                                      cusparseMatDescr descrA,
		//                                      CUdeviceptr bsrValA,
		//                                      CUdeviceptr bsrRowPtrA,
		//                                      CUdeviceptr bsrColIndA,
		//                                      int   rowBlockDim,
		//                                      int   colBlockDim,
		//                                      cusparseMatDescr descrC,
		//                                      CUdeviceptr csrValC,
		//                                      CUdeviceptr csrRowPtrC,
		//                                      CUdeviceptr csrColIndC );


		///// <summary/>
		//[DllImport(CUSPARSE_API_DLL_NAME)]
		//public static extern cusparseStatus cusparseDgebsr2csr(cusparseContext handle,
		//                                      cusparseDirection dirA,
		//                                      int mb,
		//                                      int nb,
		//                                      cusparseMatDescr descrA,
		//                                      CUdeviceptr bsrValA,
		//                                      CUdeviceptr bsrRowPtrA,
		//                                      CUdeviceptr bsrColIndA,
		//                                      int   rowBlockDim,
		//                                      int   colBlockDim,
		//                                      cusparseMatDescr descrC,
		//                                      CUdeviceptr csrValC,
		//                                      CUdeviceptr csrRowPtrC,
		//                                      CUdeviceptr csrColIndC );


		///// <summary/>
		//[DllImport(CUSPARSE_API_DLL_NAME)]
		//public static extern cusparseStatus cusparseCgebsr2csr(cusparseContext handle,
		//                                      cusparseDirection dirA,
		//                                      int mb,
		//                                      int nb,
		//                                      cusparseMatDescr descrA,
		//                                      CUdeviceptr bsrValA,
		//                                      CUdeviceptr bsrRowPtrA,
		//                                      CUdeviceptr bsrColIndA,
		//                                      int   rowBlockDim,
		//                                      int   colBlockDim,
		//                                      cusparseMatDescr descrC,
		//                                      CUdeviceptr csrValC,
		//                                      CUdeviceptr csrRowPtrC,
		//                                      CUdeviceptr csrColIndC );


		///// <summary/>
		//[DllImport(CUSPARSE_API_DLL_NAME)]
		//public static extern cusparseStatus cusparseZgebsr2csr(cusparseContext handle,
		//                                      cusparseDirection dirA,
		//                                      int mb,
		//                                      int nb,
		//                                      cusparseMatDescr descrA,
		//                                      CUdeviceptr bsrValA,
		//                                      CUdeviceptr bsrRowPtrA,
		//                                      CUdeviceptr bsrColIndA,
		//                                      int   rowBlockDim,
		//                                      int   colBlockDim,
		//                                      cusparseMatDescr descrC,
		//                                      CUdeviceptr csrValC,
		//                                      CUdeviceptr csrRowPtrC,
		//                                      CUdeviceptr csrColIndC );


		///// <summary/>
		//[DllImport(CUSPARSE_API_DLL_NAME)]
		//public static extern cusparseStatus cusparseXcsr2gebsrNnz(cusparseContext handle,
		//                                      cusparseDirection dirA,
		//                                      int m,
		//                                      int n,
		//                                      cusparseMatDescr descrA,
		//                                      CUdeviceptr csrRowPtrA,
		//                                      CUdeviceptr csrColIndA,
		//                                      cusparseMatDescr descrC,
		//                                      CUdeviceptr bsrRowPtrC,
		//                                      int rowBlockDim,
		//                                      int colBlockDim,
		//                                      CUdeviceptr nnzTotalDevHostPtr);
		///// <summary/>
		//[DllImport(CUSPARSE_API_DLL_NAME)]
		//public static extern cusparseStatus cusparseXcsr2gebsrNnz(cusparseContext handle,
		//                                      cusparseDirection dirA,
		//                                      int m,
		//                                      int n,
		//                                      cusparseMatDescr descrA,
		//                                      CUdeviceptr csrRowPtrA,
		//                                      CUdeviceptr csrColIndA,
		//                                      cusparseMatDescr descrC,
		//                                      CUdeviceptr bsrRowPtrC,
		//                                      int rowBlockDim,
		//                                      int colBlockDim,
		//                                      ref int nnzTotalDevHostPtr );

		///// <summary/>
		//[DllImport(CUSPARSE_API_DLL_NAME)]
		//public static extern cusparseStatus cusparseScsr2gebsr(cusparseContext handle,
		//                                      cusparseDirection dirA,
		//                                      int m,
		//                                      int n,
		//                                      cusparseMatDescr descrA,
		//                                      CUdeviceptr csrValA,
		//                                      CUdeviceptr csrRowPtrA,
		//                                      CUdeviceptr csrColIndA,
		//                                      cusparseMatDescr descrC,
		//                                      CUdeviceptr bsrValC,
		//                                      CUdeviceptr bsrRowPtrC,
		//                                      CUdeviceptr bsrColIndC,
		//                                      int rowBlockDim,
		//                                      int colBlockDim);

		///// <summary/>
		//[DllImport(CUSPARSE_API_DLL_NAME)]
		//public static extern cusparseStatus cusparseDcsr2gebsr(cusparseContext handle,
		//                                      cusparseDirection dirA,
		//                                      int m,
		//                                      int n,
		//                                      cusparseMatDescr descrA,
		//                                      CUdeviceptr csrValA,
		//                                      CUdeviceptr csrRowPtrA,
		//                                      CUdeviceptr csrColIndA,
		//                                      cusparseMatDescr descrC,
		//                                      CUdeviceptr bsrValC,
		//                                      CUdeviceptr bsrRowPtrC,
		//                                      CUdeviceptr bsrColIndC,
		//                                      int rowBlockDim,
		//                                      int colBlockDim);

		///// <summary/>
		//[DllImport(CUSPARSE_API_DLL_NAME)]
		//public static extern cusparseStatus cusparseCcsr2gebsr(cusparseContext handle,
		//                                      cusparseDirection dirA,
		//                                      int m,
		//                                      int n,
		//                                      cusparseMatDescr descrA,
		//                                      CUdeviceptr csrValA,
		//                                      CUdeviceptr csrRowPtrA,
		//                                      CUdeviceptr csrColIndA,
		//                                      cusparseMatDescr descrC,
		//                                      CUdeviceptr bsrValC,
		//                                      CUdeviceptr bsrRowPtrC,
		//                                      CUdeviceptr bsrColIndC,
		//                                      int rowBlockDim,
		//                                      int colBlockDim);

		///// <summary/>
		//[DllImport(CUSPARSE_API_DLL_NAME)]
		//public static extern cusparseStatus cusparseZcsr2gebsr(cusparseContext handle,
		//                                      cusparseDirection dirA,
		//                                      int m,
		//                                      int n,
		//                                      cusparseMatDescr descrA,
		//                                      CUdeviceptr csrValA,
		//                                      CUdeviceptr csrRowPtrA,
		//                                      CUdeviceptr csrColIndA,
		//                                      cusparseMatDescr descrC,
		//                                      CUdeviceptr bsrValC,
		//                                      CUdeviceptr bsrRowPtrC,
		//                                      CUdeviceptr bsrColIndC,
		//                                      int rowBlockDim,
		//                                      int colBlockDim);


		///// <summary/>
		//[DllImport(CUSPARSE_API_DLL_NAME)]
		//public static extern cusparseStatus cusparseXgebsr2gebsrNnz(cusparseContext handle,
		//                                      cusparseDirection dirA,
		//                                      int mb,
		//                                      int nb,
		//                                      int nnzb,
		//                                      cusparseMatDescr descrA,
		//                                      CUdeviceptr bsrRowPtrA,
		//                                      CUdeviceptr bsrColIndA,
		//                                      int rowBlockDimA,
		//                                      int colBlockDimA,
		//                                      cusparseMatDescr descrC,
		//                                      CUdeviceptr bsrRowPtrC,
		//                                      int rowBlockDimC,
		//                                      int colBlockDimC,
		//                                      CUdeviceptr nnzTotalDevHostPtr);

		///// <summary/>
		//[DllImport(CUSPARSE_API_DLL_NAME)]
		//public static extern cusparseStatus cusparseXgebsr2gebsrNnz(cusparseContext handle,
		//                                      cusparseDirection dirA,
		//                                      int mb,
		//                                      int nb,
		//                                      int nnzb,
		//                                      cusparseMatDescr descrA,
		//                                      CUdeviceptr bsrRowPtrA,
		//                                      CUdeviceptr bsrColIndA,
		//                                      int rowBlockDimA,
		//                                      int colBlockDimA,
		//                                      cusparseMatDescr descrC,
		//                                      CUdeviceptr bsrRowPtrC,
		//                                      int rowBlockDimC,
		//                                      int colBlockDimC,
		//                                      ref int nnzTotalDevHostPtr);

		///// <summary/>
		//[DllImport(CUSPARSE_API_DLL_NAME)]
		//public static extern cusparseStatus cusparseSgebsr2gebsr(cusparseContext handle,
		//                                      cusparseDirection dirA,
		//                                      int mb,
		//                                      int nb,
		//                                      int nnzb,
		//                                      cusparseMatDescr descrA,
		//                                      CUdeviceptr bsrValA,
		//                                      CUdeviceptr bsrRowPtrA,
		//                                      CUdeviceptr bsrColIndA,
		//                                      int rowBlockDimA,
		//                                      int colBlockDimA,
		//                                      cusparseMatDescr descrC,
		//                                      CUdeviceptr bsrValC,
		//                                      CUdeviceptr bsrRowPtrC,
		//                                      CUdeviceptr bsrColIndC,
		//                                      int rowBlockDimC,
		//                                      int colBlockDimC);


		///// <summary/>
		//[DllImport(CUSPARSE_API_DLL_NAME)]
		//public static extern cusparseStatus cusparseDgebsr2gebsr(cusparseContext handle,
		//                                      cusparseDirection dirA,
		//                                      int mb,
		//                                      int nb,
		//                                      int nnzb,
		//                                      cusparseMatDescr descrA,
		//                                      CUdeviceptr bsrValA,
		//                                      CUdeviceptr bsrRowPtrA,
		//                                      CUdeviceptr bsrColIndA,
		//                                      int rowBlockDimA,
		//                                      int colBlockDimA,
		//                                      cusparseMatDescr descrC,
		//                                      CUdeviceptr bsrValC,
		//                                      CUdeviceptr bsrRowPtrC,
		//                                      CUdeviceptr bsrColIndC,
		//                                      int rowBlockDimC,
		//                                      int colBlockDimC);


		///// <summary/>
		//[DllImport(CUSPARSE_API_DLL_NAME)]
		//public static extern cusparseStatus cusparseCgebsr2gebsr(cusparseContext handle,
		//                                      cusparseDirection dirA,
		//                                      int mb,
		//                                      int nb,
		//                                      int nnzb,
		//                                      cusparseMatDescr descrA,
		//                                      CUdeviceptr bsrValA,
		//                                      CUdeviceptr bsrRowPtrA,
		//                                      CUdeviceptr bsrColIndA,
		//                                      int rowBlockDimA,
		//                                      int colBlockDimA,
		//                                      cusparseMatDescr descrC,
		//                                      CUdeviceptr bsrValC,
		//                                      CUdeviceptr bsrRowPtrC,
		//                                      CUdeviceptr bsrColIndC,
		//                                      int rowBlockDimC,
		//                                      int colBlockDimC);


		///// <summary/>
		//[DllImport(CUSPARSE_API_DLL_NAME)]
		//public static extern cusparseStatus cusparseZgebsr2gebsr(cusparseContext handle,
		//                                      cusparseDirection dirA,
		//                                      int mb,
		//                                      int nb,
		//                                      int nnzb,
		//                                      cusparseMatDescr descrA,
		//                                      CUdeviceptr bsrValA,
		//                                      CUdeviceptr bsrRowPtrA,
		//                                      CUdeviceptr bsrColIndA,
		//                                      int rowBlockDimA,
		//                                      int colBlockDimA,
		//                                      cusparseMatDescr descrC,
		//                                      CUdeviceptr bsrValC,
		//                                      CUdeviceptr bsrRowPtrC,
		//                                      CUdeviceptr bsrColIndC,
		//                                      int rowBlockDimC,
		//                                      int colBlockDimC);

		#endregion


		#region host
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseSbsrmv(cusparseContext handle,
											cusparseDirection dirA,
											cusparseOperation transA,
											int mb,
											int nb,
											int nnzb,
											ref float alpha,
											cusparseMatDescr descrA,
											CUdeviceptr bsrValA,
											CUdeviceptr bsrRowPtrA,
											CUdeviceptr bsrColIndA,
											int blockDim,
											CUdeviceptr x,
											ref float beta,
											CUdeviceptr y);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDbsrmv(cusparseContext handle,
											cusparseDirection dirA,
											cusparseOperation transA,
											int mb,
											int nb,
											int nnzb,
											ref double alpha,
											cusparseMatDescr descrA,
											CUdeviceptr bsrValA,
											CUdeviceptr bsrRowPtrA,
											CUdeviceptr bsrColIndA,
											int blockDim,
											CUdeviceptr x,
											ref double beta,
											CUdeviceptr y);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCbsrmv(cusparseContext handle,
											cusparseDirection dirA,
											cusparseOperation transA,
											int mb,
											int nb,
											int nnzb,
											ref cuFloatComplex alpha,
											cusparseMatDescr descrA,
											CUdeviceptr bsrValA,
											CUdeviceptr bsrRowPtrA,
											CUdeviceptr bsrColIndA,
											int blockDim,
											CUdeviceptr x,
											ref cuFloatComplex beta,
											CUdeviceptr y);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZbsrmv(cusparseContext handle,
											cusparseDirection dirA,
											cusparseOperation transA,
											int mb,
											int nb,
											int nnzb,
											ref cuDoubleComplex alpha,
											cusparseMatDescr descrA,
											CUdeviceptr bsrValA,
											CUdeviceptr bsrRowPtrA,
											CUdeviceptr bsrColIndA,
											int blockDim,
											CUdeviceptr x,
											ref cuDoubleComplex beta,
											CUdeviceptr y);


		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseSbsrxmv(cusparseContext handle,
											cusparseDirection dirA,
											cusparseOperation transA,
											int sizeOfMask,
											int mb,
											int nb,
											int nnzb,
											ref float alpha,
											cusparseMatDescr descrA,
											CUdeviceptr bsrValA,
											CUdeviceptr bsrMaskPtrA,
											CUdeviceptr bsrRowPtrA,
											CUdeviceptr bsrEndPtrA,
											CUdeviceptr bsrColIndA,
											int blockDim,
											CUdeviceptr x,
											ref float beta,
											CUdeviceptr y);


		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDbsrxmv(cusparseContext handle,
											cusparseDirection dirA,
											cusparseOperation transA,
											int sizeOfMask,
											int mb,
											int nb,
											int nnzb,
											ref double alpha,
											cusparseMatDescr descrA,
											CUdeviceptr bsrValA,
											CUdeviceptr bsrMaskPtrA,
											CUdeviceptr bsrRowPtrA,
											CUdeviceptr bsrEndPtrA,
											CUdeviceptr bsrColIndA,
											int blockDim,
											CUdeviceptr x,
											ref double beta,
											CUdeviceptr y);


		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCbsrxmv(cusparseContext handle,
											cusparseDirection dirA,
											cusparseOperation transA,
											int sizeOfMask,
											int mb,
											int nb,
											int nnzb,
											ref cuFloatComplex alpha,
											cusparseMatDescr descrA,
											CUdeviceptr bsrValA,
											CUdeviceptr bsrMaskPtrA,
											CUdeviceptr bsrRowPtrA,
											CUdeviceptr bsrEndPtrA,
											CUdeviceptr bsrColIndA,
											int blockDim,
											CUdeviceptr x,
											ref cuFloatComplex beta,
											CUdeviceptr y);


		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZbsrxmv(cusparseContext handle,
											cusparseDirection dirA,
											cusparseOperation transA,
											int sizeOfMask,
											int mb,
											int nb,
											int nnzb,
											ref cuDoubleComplex alpha,
											cusparseMatDescr descrA,
											CUdeviceptr bsrValA,
											CUdeviceptr bsrMaskPtrA,
											CUdeviceptr bsrRowPtrA,
											CUdeviceptr bsrEndPtrA,
											CUdeviceptr bsrColIndA,
											int blockDim,
											CUdeviceptr x,
											ref cuDoubleComplex beta,
											CUdeviceptr y);

		#endregion
		#region device
		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseSbsrmv(cusparseContext handle,
											cusparseDirection dirA,
											cusparseOperation transA,
											int mb,
											int nb,
											int nnzb,
											CUdeviceptr alpha,
											cusparseMatDescr descrA,
											CUdeviceptr bsrValA,
											CUdeviceptr bsrRowPtrA,
											CUdeviceptr bsrColIndA,
											int blockDim,
											CUdeviceptr x,
											CUdeviceptr beta,
											CUdeviceptr y);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDbsrmv(cusparseContext handle,
											cusparseDirection dirA,
											cusparseOperation transA,
											int mb,
											int nb,
											int nnzb,
											CUdeviceptr alpha,
											cusparseMatDescr descrA,
											CUdeviceptr bsrValA,
											CUdeviceptr bsrRowPtrA,
											CUdeviceptr bsrColIndA,
											int blockDim,
											CUdeviceptr x,
											CUdeviceptr beta,
											CUdeviceptr y);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCbsrmv(cusparseContext handle,
											cusparseDirection dirA,
											cusparseOperation transA,
											int mb,
											int nb,
											int nnzb,
											CUdeviceptr alpha,
											cusparseMatDescr descrA,
											CUdeviceptr bsrValA,
											CUdeviceptr bsrRowPtrA,
											CUdeviceptr bsrColIndA,
											int blockDim,
											CUdeviceptr x,
											CUdeviceptr beta,
											CUdeviceptr y);

		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZbsrmv(cusparseContext handle,
											cusparseDirection dirA,
											cusparseOperation transA,
											int mb,
											int nb,
											int nnzb,
											CUdeviceptr alpha,
											cusparseMatDescr descrA,
											CUdeviceptr bsrValA,
											CUdeviceptr bsrRowPtrA,
											CUdeviceptr bsrColIndA,
											int blockDim,
											CUdeviceptr x,
											CUdeviceptr beta,
											CUdeviceptr y);


		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseSbsrxmv(cusparseContext handle,
											cusparseDirection dirA,
											cusparseOperation transA,
											int sizeOfMask,
											int mb,
											int nb,
											int nnzb,
											CUdeviceptr alpha,
											cusparseMatDescr descrA,
											CUdeviceptr bsrValA,
											CUdeviceptr bsrMaskPtrA,
											CUdeviceptr bsrRowPtrA,
											CUdeviceptr bsrEndPtrA,
											CUdeviceptr bsrColIndA,
											int blockDim,
											CUdeviceptr x,
											CUdeviceptr beta,
											CUdeviceptr y);


		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseDbsrxmv(cusparseContext handle,
											cusparseDirection dirA,
											cusparseOperation transA,
											int sizeOfMask,
											int mb,
											int nb,
											int nnzb,
											CUdeviceptr alpha,
											cusparseMatDescr descrA,
											CUdeviceptr bsrValA,
											CUdeviceptr bsrMaskPtrA,
											CUdeviceptr bsrRowPtrA,
											CUdeviceptr bsrEndPtrA,
											CUdeviceptr bsrColIndA,
											int blockDim,
											CUdeviceptr x,
											CUdeviceptr beta,
											CUdeviceptr y);


		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseCbsrxmv(cusparseContext handle,
											cusparseDirection dirA,
											cusparseOperation transA,
											int sizeOfMask,
											int mb,
											int nb,
											int nnzb,
											CUdeviceptr alpha,
											cusparseMatDescr descrA,
											CUdeviceptr bsrValA,
											CUdeviceptr bsrMaskPtrA,
											CUdeviceptr bsrRowPtrA,
											CUdeviceptr bsrEndPtrA,
											CUdeviceptr bsrColIndA,
											int blockDim,
											CUdeviceptr x,
											CUdeviceptr beta,
											CUdeviceptr y);


		/// <summary/>
		[DllImport(CUSPARSE_API_DLL_NAME)]
		public static extern cusparseStatus cusparseZbsrxmv(cusparseContext handle,
											cusparseDirection dirA,
											cusparseOperation transA,
											int sizeOfMask,
											int mb,
											int nb,
											int nnzb,
											CUdeviceptr alpha,
											cusparseMatDescr descrA,
											CUdeviceptr bsrValA,
											CUdeviceptr bsrMaskPtrA,
											CUdeviceptr bsrRowPtrA,
											CUdeviceptr bsrEndPtrA,
											CUdeviceptr bsrColIndA,
											int blockDim,
											CUdeviceptr x,
											CUdeviceptr beta,
											CUdeviceptr y);

		#endregion




		#endregion
	}
}
